//+------------------------------------------------------------------+
//|                                                      ProjectName |
//|                                      Copyright 2012, CompanyName |
//|                                       http://www.companyname.net |
//+------------------------------------------------------------------+
#include <Arrays\Array.mqh>
#include <Controls\TableListView.mqh>
#include <Controls\Dialog.mqh>
#include <Controls\RadioButton.mqh>
#include <Controls\CheckGroup.mqh>
#include <Controls\Button.mqh>
#include <Controls\Label.mqh>
#include <Controls\ComboBox.mqh>
#include <Controls\RadioButton.mqh>
#include <Controls\RadioGroup.mqh>
#include <RegularExpressions\Regex.mqh>
//+------------------------------------------------------------------+
//| Represents one record in the table.                              |
//+------------------------------------------------------------------+
class OptimizatorRecord : public IComparable
  {
private:
   double            m_values[];
public:
   //--- Constructors:
   //+------------------------------------------------------------------+
   //| Constructors with parameters.                                    |
   //+------------------------------------------------------------------+
                     OptimizatorRecord(double &values[])
     {
      ArrayCopy(m_values,values);
     }
   //--- Methods:
   //+------------------------------------------------------------------+
   //| Operator ([]).                                                   |
   //+------------------------------------------------------------------+
   double operator[](const int i)
     {
      //--- return result     
      return (m_values[i]);
     }
  };
//+------------------------------------------------------------------+
//| Represents the result after the processing table.                |
//+------------------------------------------------------------------+
class OptimizatorResultParsing
  {
private:
   string            m_file_name;
   int               m_handel;
   List<string>*m_column_names;
   List<OptimizatorRecord*>*m_rows;
public:
   //--- Constructors:
   //+------------------------------------------------------------------+
   //| Constructor with parameters.                                     |
   //+------------------------------------------------------------------+
                     OptimizatorResultParsing(const string path)
     {
      m_file_name=path;
      m_handel=FileOpen(path,FILE_READ|FILE_ANSI);
      m_rows=new List<OptimizatorRecord*>();
      m_column_names=new List<string>();
      Regex *rgx=new Regex("(<Data ss:Type=\")(\\w+)(\">)(.*?)(</Data>)");
      bool title_create=false;
      while(!FileIsEnding(m_handel))
        {
         string str=FileReadString(m_handel);
         Match *match=rgx.Match(str);
         GroupCollection *groups=match.Groups();
         string type=groups[2].Value();
         if(type=="String")
           {
            m_column_names.Add(groups[4].Value());
            title_create=true;
           }
         else if(title_create)
           {
            break;
           }
         delete match;
        }
      int count=m_column_names.Count();
      double row[];
      ArrayResize(row,count);
      ZeroMemory(row);
      int i=0;
      while(!FileIsEnding(m_handel))
        {
         string str=FileReadString(m_handel);
         Match *match=rgx.Match(str);
         GroupCollection *groups=match.Groups();
         string type=groups[2].Value();
         if(type=="Number")
           {
            string s0=groups[0].Value();
            string s1=groups[1].Value();
            string s2=groups[2].Value();
            string s3=groups[3].Value();
            string s4=groups[4].Value();
            row[i]=StringToDouble(groups[4].Value());
            if(i<count-1)
              {
               i++;
              }
            else
              {
               i=0;
               m_rows.Add(new OptimizatorRecord(row));
              }
           }
         delete match;
        }
      FileClose(m_handel);
      delete rgx;
      Regex::ClearCache();
     }
   //--- Destructors:
   //+------------------------------------------------------------------+
   //| Destructor without parameters.                                   |
   //+------------------------------------------------------------------+
                    ~OptimizatorResultParsing()
     {
      delete m_column_names;
      for(int i=0; i<m_rows.Count(); i++)
        {
         delete m_rows[i];
        }
      delete m_rows;
     }
   //--- Methods:
   //+------------------------------------------------------------------+
   //| Gets the column count.                                           |
   //+------------------------------------------------------------------+
   int ColumnCount()
     {
      //--- return result
      return (m_column_names.Count());
     }
   //+------------------------------------------------------------------+
   //| Gets the list of column names by index.                          |
   //+------------------------------------------------------------------+
   string GetColumnName(const int i)
     {
      //--- return result
      return (m_column_names[i]);
     }
   //+------------------------------------------------------------------+
   //| Gets the list of all records of the table.                       |
   //+------------------------------------------------------------------+
   List<OptimizatorRecord*>*GetRecords()
     {
      //--- return result
      return (m_rows);
     }
   //+------------------------------------------------------------------+
   //| Gets the list of column names.                                   |
   //+------------------------------------------------------------------+
   List<string>*GetColumnNames()
     {
      //--- return result
      return (m_column_names);
     }
  };
//+------------------------------------------------------------------+
//| Filter.                                                          |
//+------------------------------------------------------------------+
class Filter : public IComparable
  {
private:
   int               m_index;
   double            m_min;
   double            m_max;
public:
   //--- Constructors:
   //+------------------------------------------------------------------+
   //| Constructor with parameters.                                     |
   //+------------------------------------------------------------------+
                     Filter(const int index,const double min,const double max)
     {
      m_index=index;
      m_min=min;
      m_max=max;
     }
   //--- Methods:
   //+------------------------------------------------------------------+
   //| Gets the index of column.                                        |
   //+------------------------------------------------------------------+
   int Index()
     {
      //--- return result
      return (m_index);
     }
   //+------------------------------------------------------------------+
   //| Gets the min value.                                              |
   //+------------------------------------------------------------------+
   double Min()
     {
      //--- return result
      return (m_min);
     }
   //+------------------------------------------------------------------+
   //| Gest the max value.                                              |
   //+------------------------------------------------------------------+
   double Max()
     {
      //--- return result
      return (m_max);
     }
  };
//+------------------------------------------------------------------+
//| TradeHistoryView.                                                |
//+------------------------------------------------------------------+
class OptimizatorResultView : public CAppDialog
  {
private:
   OptimizatorResultParsing *m_result;

   string            m_results_names[10];
   string            m_parameters_names[10];

   CRadioButton      m_radio_buttons[10];

   CLabel            m_label2;
   CEdit             m_headers[10];

   CTableListView    m_table;

   CLabel            m_label1;
   CRadioGroup       m_select_table;

   CLabel            m_label3;
   CLabel            m_label4;
   CLabel            m_label5;
   CButton           m_confirm;
   CButton           m_add_filter;
   CButton           m_delete_filter;
   CEdit             m_min;
   CEdit             m_max;
   CEdit             m_line;

   CComboBox         m_columns;

   CListView         m_filters_view;

   List<string>m_active;
   bool              m_max_put;
   bool              m_min_put;
   CMatrix<double>m_tadle_value;
   List<Filter*>m_filters;
public:
   //--- handlers of the dependent controls events
   //+------------------------------------------------------------------+
   //| Event handler.                                                   |
   //+------------------------------------------------------------------+
   void OnChangeCRadioGroup()
     {
      if(m_select_table.Value()==1)
        {
         SetResultsView();
        }
      if(m_select_table.Value()==2)
        {
         SetParametersView();
        }
     }
   //+------------------------------------------------------------------+
   //| Event handler.                                                   |
   //+------------------------------------------------------------------+
   void OnClickButtonDeleteFilter()
     {
      m_filters_view.ItemDelete(0);
      delete m_filters[0];
      m_filters.RemoveAt(0);
      PrintTable();
     }
   //+------------------------------------------------------------------+
   //| Event handler.                                                   |
   //+------------------------------------------------------------------+
   void OnClickButtonAddFilter()
     {
      double min = StringToDouble(m_min.Text());
      double max = StringToDouble(m_max.Text());
      string column_name=m_columns.Select();
      string filter=column_name+" : "+DoubleToString(min,3)+" - "+DoubleToString(max,3);
      if(min<=max && !m_filters_view.SelectByText(filter))
        {
         m_filters_view.AddItem(filter);
         m_filters.Add(new Filter(m_active.IndexOf(column_name),min,max));
        }
      PrintTable();
     }
   //+------------------------------------------------------------------+
   //| Event handler.                                                   |
   //+------------------------------------------------------------------+
   void OnChangeComboBoxColumn()
     {
      m_max.Visible(true);
      m_min.Visible(true);
      m_add_filter.Visible(true);
      m_delete_filter.Visible(true);
      m_label4.Visible(true);
      m_label3.Visible(true);
      m_label2.Visible(true);
      m_filters_view.Visible(true);
     }
   //+------------------------------------------------------------------+
   //| Event handler.                                                   |
   //+------------------------------------------------------------------+
   void OnClickButtonConfirm()
     {
      m_filters_view.ItemsClear();
      m_active.Clear();
      CreateHeaders();
      m_table.VScrolled(false);
      m_table.Destroy();
      CreateTable();
      PrintTable();
      m_columns.ItemsClear();
      SetComboBox();
     }
   //+------------------------------------------------------------------+
   //| Event Handling.                                                  |
   //+------------------------------------------------------------------+   
   EVENT_MAP_BEGIN(OptimizatorResultView)
   ON_EVENT(ON_CHANGE,m_select_table,OnChangeCRadioGroup)
   ON_EVENT(ON_CLICK,m_confirm,OnClickButtonConfirm)
   ON_EVENT(ON_CHANGE,m_columns,OnChangeComboBoxColumn)
   ON_EVENT(ON_CLICK,m_add_filter,OnClickButtonAddFilter)
   ON_EVENT(ON_CLICK,m_delete_filter,OnClickButtonDeleteFilter)
   EVENT_MAP_END(CAppDialog)
   //--- Destructors:
   //+------------------------------------------------------------------+
   //| Destructor withou parameters.                                    |
   //+------------------------------------------------------------------+
                    ~OptimizatorResultView()
     {
      for(int i=0; i<m_filters.Count(); i++)
        {
         delete m_filters[i];
        }
      delete m_result;
     }
   //--- Methods:
   //+------------------------------------------------------------------+
   //| Creat a control.                                                 |
   //+------------------------------------------------------------------+
   bool Create(OptimizatorResultParsing *result)
     {
      m_result=result;
      ZeroMemory(m_parameters_names);
      for(int i=0; i<10; i++)
        {
         m_results_names[i]=result.GetColumnName(i);
        }
      for(int i=0; i<10; i++)
        {
         m_parameters_names[i]=result.GetColumnName(i+10);
        }
      CAppDialog::Create(0,"Tester Optimizator Results",0,15,15,1300,600);
      CreateCheckGroup();
      CreateLabels();
      CreateRadioButtons();
      CreateButtonConfirm();
      CreateComboBox();
      CreateRange();
      CreateButtonAddFilter();
      CreateFiltersView();
      CreateButtonDeleteFilter();
      DefaultView();
      return (true);
     }
   //+------------------------------------------------------------------+
   //| Cereate a combo box controls.                                    |
   //+------------------------------------------------------------------+
   bool CreateCheckGroup()
     {
      int x1 = 20;
      int y1 = 30;
      int x2 = 200;
      int y2 = 70;
      m_select_table.Create(0,"CheckGroup1",0,x1,y1,x2,y2);
      m_select_table.AddItem("Tester results table.",1);
      m_select_table.AddItem("Input parameters table.",2);
      Add(m_select_table);
      return (true);
     }
   //+------------------------------------------------------------------+
   //| Create labels.                                                   |
   //+------------------------------------------------------------------+
   bool CreateLabels()
     {
      m_label1.Create(0,"Lable1",0,20,10,100,30);
      m_label1.Text("Select the table:");
      Add(m_label1);
      m_label2.Create(0,"Lable2",0,20,240,220,260);
      m_label2.Text("Records in table: ");
      Add(m_label2);
      m_line.Create(0,"Edit3",0,20,230,this.Width()-20,233);
      m_line.ColorBackground(clrDarkGray);
      m_line.ReadOnly(true);
      m_line.Text(NULL);
      Add(m_line);
      //--- return result
      return (true);
     }
   //+------------------------------------------------------------------+
   //| Create radio buttons.                                            |
   //+------------------------------------------------------------------+
   bool CreateRadioButtons()
     {
      int x1 = 20;
      int y1 = 80;
      int x2 = 220;
      int y2 = 100;
      for(int i=0; i<10; i++)
        {
         m_radio_buttons[i].Create(0,"RadioButton"+IntegerToString(i),0,x1,y1,x2,y2);
         y1=y2;
         y2+=20;
         if(i==4)
           {
            x1=240;
            y1=80;
            x2=440;
            y2=100;
           }
         Add(m_radio_buttons[i]);
        }
      return (true);
     }
   //+------------------------------------------------------------------+
   //|                                                                  |
   //+------------------------------------------------------------------+
   bool CreateHeaders()
     {
      for(int i=0; i<ArraySize(m_headers); i++)
        {
         m_headers[i].Destroy();
        }
      m_active.Clear();
      for(int i=0; i<10; i++)
        {
         if(m_radio_buttons[i].State())
           {
            m_active.Add(m_radio_buttons[i].Text());
           }
        }
      if(m_active.Count()==0)
        {
         m_active.Add(NULL);
        }
      int step=(this.Width()-40)/m_active.Count();
      int x1=20;
      int x2=x1+step;
      for(int i=0; i<m_active.Count(); i++)
        {
         m_headers[i].Create(0,"Header"+IntegerToString(i),0,x1,260,x2,280);
         m_headers[i].ColorBackground(clrDarkGray);
         m_headers[i].TextAlign(ALIGN_CENTER);
         m_headers[i].Text(m_active[i]);
         Add(m_headers[i]);
         x1=x2;
         x2+=step;
        }
      return (true);
     }
   //+------------------------------------------------------------------+
   //| Create button.                                                   |
   //+------------------------------------------------------------------+
   bool CreateButtonConfirm()
     {
      m_confirm.Create(0,"Button1",0,20,190,200,220);
      m_confirm.Text("Refresh table");
      Add(m_confirm);
      return (true);
     }
   //+------------------------------------------------------------------+
   //| Create combo box.                                                |
   //+------------------------------------------------------------------+
   bool CreateComboBox()
     {
      m_label5.Create(0,"Lable5",0,470,15,650,35);
      m_label5.Text("Select the column to filter.");
      Add(m_label5);
      m_columns.Create(0,"ComboBox1",0,470,40,650,60);
      Add(m_columns);
      return (true);
     }
   //+------------------------------------------------------------------+
   //| Create area to select range for filter.                          |
   //+------------------------------------------------------------------+
   bool CreateRange()
     {
      m_label3.Create(0,"Label3",0,470,60,650,80);
      m_label3.Text("Set range:");
      Add(m_label3);
      m_min.Create(0,"CEdit1",0,470,80,550,100);
      Add(m_min);
      m_max.Create(0,"CEdit2",0,570,80,650,100);
      Add(m_max);
      m_label4.Create(0,"Label4",0,555,70,565,90);
      m_label4.Text("-");
      m_label4.FontSize(20);
      Add(m_label4);
      return (true);
     }
   //+------------------------------------------------------------------+
   //| Create button.                                                   |
   //+------------------------------------------------------------------+
   bool CreateButtonAddFilter()
     {
      m_add_filter.Create(0,"Button2",0,470,110,650,140);
      m_add_filter.Text("Add filter");
      Add(m_add_filter);
      return (true);
     }
   //+------------------------------------------------------------------+
   //| Create button.                                                   |
   //+------------------------------------------------------------------+
   bool CreateButtonDeleteFilter()
     {
      m_delete_filter.Create(0,"Button3",0,470,150,650,180);
      m_delete_filter.Text("Delete filter");
      Add(m_delete_filter);
      return (true);
     }
   //+------------------------------------------------------------------+
   //| Create list view.                                                |
   //+------------------------------------------------------------------+
   bool CreateFiltersView()
     {
      m_label5.Create(0,"Label5",0,690,15,750,35);
      m_label5.Text("Current filters:");
      Add(m_label5);
      m_filters_view.Create(0,"ListView1",0,690,40,1150,40+(10/2)*20+40);
      Add(m_filters_view);
      return (true);
     }
   //+------------------------------------------------------------------+
   //| Create table.                                                    |
   //+------------------------------------------------------------------+
   bool CreateTable()
     {
      ushort sizes[];
      ArrayResize(sizes,m_active.Count());
      ArrayFill(sizes,0,m_active.Count(),(ushort)((this.Width()-50)/m_active.Count()));
      m_table.Create(0,"Table 1",0,20,280,this.Width()-20,540,(uchar)m_active.Count(),sizes);
      Add(m_table);
      return (true);
     }
   //+------------------------------------------------------------------+
   //|                                                                  |
   //+------------------------------------------------------------------+
   void DefaultView()
     {
      for(int i=0; i<10; i++)
        {
         m_radio_buttons[i].Text(m_results_names[i]);
        }
      m_radio_buttons[0].State(true);
      m_radio_buttons[1].State(true);
      m_radio_buttons[4].State(true);
      m_radio_buttons[5].State(true);
      m_select_table.Value(1);
      CreateHeaders();
      PrintTable();
      SetComboBox();
     }
   //+------------------------------------------------------------------+
   //|                                                                  |
   //+------------------------------------------------------------------+
   void SetComboBox()
     {
      m_columns.ItemsClear();
      for(int i=0; i<m_active.Count(); i++)
        {
         m_columns.AddItem(m_active[i],i);
        }
     }
   //+------------------------------------------------------------------+
   //| Refresh table and filters.                                       |
   //+------------------------------------------------------------------+
   void Refresh()
     {
      for(int i=0; i<m_filters.Count(); i++)
        {
         delete m_filters[i];
        }
      m_filters.Clear();
      for(int i=0; i<10; i++)
        {
         m_radio_buttons[i].Visible(true);
         m_radio_buttons[i].State(false);
        }
      CreateHeaders();
      m_table.VScrolled(false);
      m_table.Destroy();
      CreateTable();
      m_label2.Text("Records in table: ");
     }
   //+------------------------------------------------------------------+
   //| Print the table.                                                 |
   //+------------------------------------------------------------------+
   void PrintTable()
     {
      int table_count=0;
      m_table.Destroy();
      CreateTable();
      m_table.VScrolled(false);
      int indices[];
      ArrayResize(indices,m_active.Count());
      for(int i=0; i<ArraySize(indices); i++)
        {
         indices[i]=m_result.GetColumnNames().IndexOf(m_active[i]);
        }
      for(int i=0; i<m_result.GetRecords().Count(); i++)
        {
         string row[];
         long values[];
         ArrayResize(row,ArraySize(indices));
         ArrayResize(values,ArraySize(indices));
         ZeroMemory(values);
         bool add = true;
         for(int j=0; j<ArraySize(row); j++)
           {
            double value=m_result.GetRecords()[i][indices[j]];
            if(ChecksValue(value,j))
              {
               row[j]=(m_headers[j].Text()=="Pass") ? DoubleToString(value,0) : DoubleToString(value,3);
              }
            else
              {
               add=false;
               break;
              }
           }
         if(add)
           {
            table_count++;
            m_table.AddItem(row,values);
           }
        }
      m_label2.Text("Records in table: "+IntegerToString(table_count));
     }
   //+------------------------------------------------------------------+
   //| Checks value.                                                    |
   //+------------------------------------------------------------------+
   bool ChecksValue(const double value,const int column_index)
     {
      for(int i=0; i<m_filters.Count(); i++)
        {
         if(column_index==m_filters[i].Index() && 
            (m_filters[i].Max()<value || m_filters[i].Min()>value))
           {
            return (false);
           }
        }
      return (true);
     }
   //+------------------------------------------------------------------+
   //|                                                                  |
   //+------------------------------------------------------------------+
   void SetResultsView()
     {
      Refresh();
      for(int i=0; i<10; i++)
        {
         m_radio_buttons[i].Text(m_results_names[i]);
        }
     }
   //+------------------------------------------------------------------+
   //|                                                                  |
   //+------------------------------------------------------------------+
   void SetParametersView()
     {
      Refresh();
      for(int i=0; i<10; i++)
        {
         m_radio_buttons[i].Text(m_parameters_names[i]);
         if(m_parameters_names[i]==NULL)
           {
            m_radio_buttons[i].Visible(false);
           }
        }
     }
  };
//+------------------------------------------------------------------+
